package com.perforce.workshop.tjuricek.p4oauth

import org.gradle.api.DefaultTask
import org.gradle.api.tasks.TaskAction

/**
 *
 */
class CreateKeystoreTask extends DefaultTask {

    @TaskAction
    def create() {
        if (!keystoreFile.parentFile.exists()) {
            keystoreFile.parentFile.mkdirs()
        }

        // I'm not sure if this should use the 'getState' method or not
        if (keystoreFile.exists()) {
            setDidWork(false);
            return;
        }

        ProcessBuilder pb = new ProcessBuilder(
                "keytool", "-genkeypair", "-dname",
                "cn=${hostname}, ou=Engineering, o=Perforce, c=US",
                "-alias", "business", "-keypass", keyPass,
                "-keystore", keystoreFile.absolutePath,
                "-storepass", storePass, "-validity", "180"
        );

        def proc = pb.start()
        int exitVal = proc.waitFor()
        if (exitVal != 0) {
            logger.warn("keytool cmd: " + pb.command().join(" "));
            throw new IllegalStateException("keytool failed:\n" +
                    proc.getErrorStream().getText("utf8"))
        }
    }

    File getKeystoreFile() {
        return project.file("${project.buildDir}/keystore");
    }

    String getKeyPass() {
        return "password"
    }

    String getStorePass() {
        return "password"
    }

    String getHostname() {
        String hostname = null
        if (System.getProperty("COMPUTERNAME") != null) {
            hostname = System.getProperty("COMPUTERNAME"); // Windows
        } else if (System.getProperty("HOSTNAME") != null) {
            hostname = System.getProperty("HOSTNAME"); // Unix, sometimes
        }
        if (hostname == null) {
            try {
                hostname = InetAddress.getLocalHost().getHostName();
            } catch (UnknownHostException e) {
                // Ignore, we just don't know it by default
            }
        }
        return hostname
    }
}
